package com.jetbyte.cache8;

import java.util.Map;
import java.util.TreeMap;
import java.util.Iterator;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;

public class CacheManager
{
   synchronized public static Cache getInstance(String name) 
   {
      Cache cache = get(name);

      if (cache == null)
      {  
         cache = createNewCache(name);  
         
         caches.put(new Cache.Key(name), cache);     
      }

      return cache;
   }

   synchronized public static Cache get(String name)
   {
      Cache cache = null;

      key.setKey(name);

      Object obj = caches.get(key);
      
      if (obj != null)
      {
         cache = (Cache)obj;
      }    
      
      return cache;
   }

   public static Iterator iterator()
   {
      return caches.values().iterator();
   }

   public static int numCaches()
   {
      return caches.size();
   }

   private static Cache createNewCache(String name) 
   {
      String cacheClassName = System.getProperties().getProperty(CacheManager.class.getName() + "." + name + ".CacheClass");
      
      if (cacheClassName == null)
      {
         cacheClassName = System.getProperties().getProperty(CacheManager.class.getName() + ".DefaultCacheClass");
      }   

      Cache cache = null;

      if (cacheClassName != null)
      {
         try
         {
            cache = constructCache(getConstructorForCacheClass(cacheClassName), name);
         }
         catch(Exception e)
         {
            e.printStackTrace();
         }
      }

      if (cache == null)
      {
         cache = new SimpleCache(name);
      }
         
      return cache;
   }

   private static Constructor getConstructorForCacheClass(String name) throws ClassNotFoundException, NoSuchMethodException
   {
      Class cacheClass = Class.forName(name);

      Class[] argClasses = new Class[1];
      argClasses[0] = String.class;
         
      return cacheClass.getDeclaredConstructor(argClasses);
   }

   private static Cache constructCache(Constructor ctor, String name) throws InstantiationException, IllegalAccessException, InvocationTargetException
   {
      Object[] argObjs = new Object[1];
      argObjs[0] = name;

      return (Cache)ctor.newInstance(argObjs);
   }

   private static Cache.Key key = new Cache.Key();

   private static Map caches = new TreeMap();
}
