package com.jetbyte.cache8;

import org.omg.CORBA.ORB;
import org.omg.CORBA.COMM_FAILURE;
import org.omg.CORBA.SystemException;

import org.omg.CORBA.ORBPackage.InvalidName;

import org.omg.CosNaming.NamingContext;
import org.omg.CosNaming.NamingContextHelper;
import org.omg.CosNaming.NamingContextPackage.AlreadyBound;

import org.omg.CosNaming.NameComponent;

import org.omg.PortableServer.POA;
import org.omg.PortableServer.POAHelper;

import java.util.Properties;

import java.io.InputStream;
import java.io.FileInputStream;

public class Server 
{
   public static void main(String[] args) 
   {
      loadProperties("server.properties");

      try
      {
         ORB orb = ORB.init( args, null );

         POA poa = POAHelper.narrow(orb.resolve_initial_references("RootPOA"));

         poa.the_POAManager().activate();

         CacheTestImpl test = new CacheTestImpl();

         org.omg.CORBA.Object obj = poa.servant_to_reference(test);

         if (!BindObjectToName(orb, obj, "JetByte Test", "CacheTest"))
         {
            System.out.println("Failed to bind object to name");  
            System.exit(1);
         }

         orb.run();
      }
      catch(Exception e) 
      {
         e.printStackTrace();
      }
   
      System.exit(0);
   }

   private static boolean BindObjectToName(
      ORB orb, 
      org.omg.CORBA.Object objref,
      String contextName,
      String objectName) throws Exception
   {
      boolean ok = false;

      NamingContext rootContext = null;

      try 
      {
         org.omg.CORBA.Object obj = orb.resolve_initial_references("NameService");

         rootContext = NamingContextHelper.narrow(obj);

         if (rootContext == null) 
         {
            System.out.println("Failed to narrow the root naming context.");
            return false;
         }
      }
      catch(InvalidName e) 
      {
         System.out.println("Service required is invalid [does not exist].");
         return false;
      }

      try 
      {

         NameComponent[] context_name = new NameComponent[1];
         context_name[0] = new NameComponent(contextName, "JetByte Context"); 

         NamingContext testContext = null;
 
         try 
         {
            testContext = rootContext.bind_new_context(context_name);
         }
         catch(AlreadyBound e) 
         {
            org.omg.CORBA.Object obj  = rootContext.resolve(context_name);
      
            testContext = NamingContextHelper.narrow(obj);

            if (testContext == null) 
            {
               System.out.println("Failed to narrow naming context.");
               return false;
            }
         }


         // Bind the objref to the name in the testContext:

         NameComponent[] object_name = new NameComponent[1];
         object_name[0] = new NameComponent(objectName, "Object"); 

         try 
         {
            testContext.bind(object_name, objref);
            ok = true;
         }
         catch(AlreadyBound e) 
         {
            testContext.rebind(object_name, objref);
            ok = true;
         }
      }
      catch(COMM_FAILURE e) 
      {
         System.out.println("System exception: COMM_FAILURE");
      }
      catch(SystemException e1) 
      {
         System.out.println("System exception:");
      }

      return ok;
   }

   public static void loadProperties(String propertiesFile)
   {
      final Properties props = new Properties(System.getProperties());

      InputStream stream = null;

      try
      {
         stream = new FileInputStream(propertiesFile);
      
         props.load(stream);
      }
      catch (Exception e)
      {
         e.printStackTrace();
      }
      finally 
      {
         try
         {
            if (stream != null)
            {
               stream.close();
            }
         }
         catch(Exception e)
         {
            e.printStackTrace();
         }
      }

      System.setProperties(props);
   }
}
